@@ -28,11 +28,12 @@
 This module provides a listener which listens to the events triggered while
 a campaign is being parsed, so as to build its representation.
 """
+
 from collections import defaultdict
-from typing import Union, List, Tuple
+from typing import Any, List, Tuple, Union
 
 from metrics.core.builder import CampaignBuilder
-from metrics.core.model import Model
+from metrics.core.model import Campaign
 
 
 class KeyMapping:
@@ -45,19 +46,20 @@ class KeyMapping:
         """
         Creates a new KeyMapping.
         """
-        self._dict = {}
+        self._dict_representation = {}
 
-    def __setitem__(self, scalpel_key: str, campaign_key: Union[str, List[str]]) -> None:
+    def __setitem__(self, scalpel_key: str,
+                    campaign_key: Union[str, List[str]]) -> None:
         """
         Maps a key as defined in the campaign to that expected by Scalpel.
 
-        :param campaign_key: The key defined in the campaign.
         :param scalpel_key: The key expected by Scalpel.
+        :param campaign_key: The key defined in the campaign.
         """
         if isinstance(campaign_key, str):
-            self._dict[scalpel_key] = [campaign_key]
+            self._dict_representation[scalpel_key] = [campaign_key]
         else:
-            self._dict[scalpel_key] = campaign_key
+            self._dict_representation[scalpel_key] = campaign_key
 
     def __getitem__(self, campaign_key: str) -> Tuple[str, int]:
         """
@@ -65,14 +67,21 @@ class KeyMapping:
 
         :param campaign_key: The key defined in the campaign.
 
-        :return: The key expected by Scalpel, or the given key if no mapping is
-                 set for this key.
+        :return: A tuple having, as first element, the key expected by Scalpel
+                 or the given key if no mapping is set for this key and, as
+                 second element, the number of values expected for this key.
         """
-        for k, v in self._dict.items():
-            if campaign_key in v:
-                return k, len(v)
+        for key, value in self._dict_representation.items():
+            if campaign_key in value:
+                return key, len(value)
         return campaign_key, 1
 
+    def get_sorted_keys(self, scalpel_key: str):
+        keys = self._dict_representation.get(scalpel_key)
+        if keys is None:
+            return [scalpel_key]
+        return keys
+
 
 class CampaignParserListener:
     """
@@ -87,14 +96,16 @@ class CampaignParserListener:
         self._campaign_builder = None
         self._input_set_builder = None
         self._current_builder = None
-        self._pending_keys = defaultdict(list)
+        self._pending_keys = defaultdict(dict)
+        self._current_experiment = False
 
-    def add_key_mapping(self, scalpel_key: str, campaign_key: Union[str, List[str]]) -> None:
+    def add_key_mapping(self, scalpel_key: str,
+                        campaign_key: Union[str, List[str]]) -> None:
         """
         Maps a key as defined in the campaign to that expected by Scalpel.
 
-        :param campaign_key: The key defined in the campaign.
         :param scalpel_key: The key expected by Scalpel.
+        :param campaign_key: The key defined in the campaign.
         """
         self._key_mapping[scalpel_key] = campaign_key
 
@@ -119,7 +130,8 @@ class CampaignParserListener:
 
     def end_experiment_ware(self) -> None:
         """
-        Notifies this listener that the current experiment-ware has been fully parsed.
+        Notifies this listener that the current experiment-ware has been
+        fully parsed.
         """
         self._current_builder = self._campaign_builder
 
@@ -154,29 +166,57 @@ class CampaignParserListener:
         Notifies this listener that a new experiment is going to be parsed.
         """
         self._current_builder = self._campaign_builder.add_experiment_builder()
+        self._current_experiment = True
 
     def end_experiment(self) -> None:
         """
-        Notifies this listener that the current experiment has been fully parsed.
+        Notifies this listener that the current experiment has been
+        fully parsed.
         """
         self._current_builder = self._campaign_builder
+        self._current_experiment = False
 
-    def log_data(self, key: str, value: str) -> None:
+    def log_data(self, key: str, value: Any) -> None:
         """
         Notifies this listener about data that has been read.
-        This data is set to the element of the campaign that is currently being built.
+        This data is set to the element of the campaign that is currently
+        being built.
 
         :param key: The key identifying the read data.
         :param value: The value that has been read.
         """
+        # Adding the read value.
         scalpel_key, nb = self._key_mapping[key]
-        self._pending_keys[scalpel_key].append(value)
-        if len(self._pending_keys[scalpel_key]) == nb:
-            values = ['' if v is None else v for v in self._pending_keys[scalpel_key]]
-            self._current_builder[scalpel_key] = ' '.join(values)
+        read_values = self._pending_keys[scalpel_key]
+        read_values[key] = str(value)
+
+        # If all the values of the mapping have been read, we can commit them.
+        if len(read_values) == nb:
+            values = ['' if read_values[v] is None else read_values[v] for v in
+                      self._key_mapping.get_sorted_keys(scalpel_key)]
+
+            name = ' '.join(values)
+            self._current_builder[scalpel_key] = name
+
+            if self._current_experiment:
+                if scalpel_key == 'experiment_ware' and not self._campaign_builder.has_experiment_ware_with_name(name):
+                    xp_ware_builder = self._campaign_builder.add_experiment_ware_builder()
+                    self.create_on_the_fly(xp_ware_builder, read_values, 'name', name)
+                if scalpel_key == 'input' and not self._campaign_builder.has_input_with_path(name):
+                    if self._input_set_builder is None:
+                        self._input_set_builder = self._campaign_builder.add_input_set_builder()
+                        self._input_set_builder['name'] = 'auto_name'
+                    input_builder = self._input_set_builder.add_input_builder()
+                    self.create_on_the_fly(input_builder, read_values, 'path', name)
             del self._pending_keys[scalpel_key]
 
-    def get_campaign(self) -> Model:
+    @staticmethod
+    def create_on_the_fly(builder, read_values, key, name):
+        for k, v in read_values.items():
+            builder[k] = v
+        builder[key] = name
+
+    def get_campaign(self) -> Campaign:
         """
         Gives the campaign that has been read.
 
